local LibQTip = LibStub('LibQTip-1.0');
local tec = TeamEquipCompare;
local tectip;
local padding = 10;

-- TODO: incorporate the TEC_DataIsComplete(unit) method
function TEC_OnTooltipSetItem(gametip, ...)
	if (TEC_TooltipIsActive()) then
		local ttData = TEC_GetTooltipItemData(gametip);

		if (ttData.slot and ttData.slot.ids) then
			local t = TEC_GetCompareDataByRow(ttData);
			local lineidx = nil;
		
			if (#t > 0) then
				tectip = LibQTip:Acquire("TECTooltip", t.cols);

				for ii = 1, #t do
					if (t[ii].type == "Header") then
						lineidx = tectip:AddHeader();
					elseif (t[ii].type == "Separator") then
						lineidx = tectip:AddSeparator(t[ii].height, t[ii].r, t[ii].g, t[ii].b);
					elseif (t[ii].type == "Spacer") then
						lineidx = tectip:AddLine(" ");
					else
						lineidx = tectip:AddLine();
					end

					if (t[ii]["type"] ~= "Separator" and t[ii].type ~= "Spacer") then
						for jj = 1, t.cols do
							if (t[ii][jj] ~= nil) then
								tectip:SetCell(lineidx, jj, t[ii][jj].text, nil, t[ii][jj].align, t[ii][jj].colspan, nil, padding, padding);
							end
						end
					end
				end

				-- anchor qtip tooltip to GameTooltip and show
				tectip:SmartAnchorTo(GameTooltip);
				tectip:Show();
			end
		end
	end
end

function TEC_OnTooltipCleared(...)
	if (tectip ~= nil) then
		LibQTip:Release(tectip)
		tectip = nil
	end
end

function TEC_TooltipIsActive()
	-- must be in a group
	if (not IsInGroup()) then return false; end
	
	-- check if we require ctrl key
	if (not tec.db.global.requireCtrlKey) then return true; end
	
	-- otherwise, check if ctrl key is pressed
	if (IsControlKeyDown()) then return true; end

	-- if we got here, it's not active
	return false;
end


-- TODO: refactor this function?
function TEC_GetCompareDataByRow(ttData)
	-- initialize compare data table
	local t = { };
	t[1] = { }; -- title row
	t[2] = { type = "Separator" };
	t[3] = { type = "Spacer" };
	t[4] = { type = "Header" }; -- unit name row
	t[5] = { type = "Separator", r = 0.75, g = 0.75, b = 0.75 };

	local colidx = 0;
	local rowidx = 0;
	local rowstart = #t;
--	local sortedNames = tec_keys(tec.party, true);
	local sortedNames = tec_keys(tec.party, false);

	-- add column headers
	for _, unitName in pairs(sortedNames) do
		if (not TEC_DataIsComplete(tec.party[unitName])) then return { } end;

		colidx = colidx + 1;

		local unit = tec.party[unitName];
		local headerText = format("|c%s%s|r |cfff0c501[%d]|r", TEC_ClassColor_Lookup[unit.class], unit.name, unit.ilevel.equipped);
		t[4][colidx] = { text = headerText, align = "CENTER" };
	end


	-- loop thru all the slots
	local slotCount = #ttData.slot.ids
	for slotidx = 1, slotCount do
		colidx = 0;

		-- add spacer row between items
		if (slotidx > 1) then
			rowstart = rowstart + 1;
			t[rowstart] = { type = "Spacer" };
		end
		
		-- loop thru all team members
		for _, unitName in pairs(sortedNames) do
			local unit = tec.party[unitName]

			rowidx = rowstart + 1;
			colidx = colidx + 1;

			-- get compare data for individual unit
			local data = TEC_GetSlotCompareData(unit, ttData, slotCount, slotidx);
			if (data.text) then
				-- add item header
				if (t[rowidx] == nil) then t[rowidx] = { } end
				t[rowidx][colidx] = { text = data.text, align = "CENTER" };
			
				if (#data.stats > 0) then
					local classOptions = TEC_Class_Lookup[unit.class];
					local isUsable, msg = TEC_IsItemUsable(unit, classOptions, ttData.reqLevel, ttData.class, ttData.subclass);

					if (isUsable) then
						-- add stat lines
						for ii = 1, #data.stats do
							-- add stat item
							rowidx = rowidx + 1;
							if (t[rowidx] == nil) then t[rowidx] = {}; end
							t[rowidx][colidx] = { text = data.stats[ii], align = "LEFT" };
						end
					else
						-- unusable item
						rowidx = rowidx + 1;
						if (t[rowidx] == nil) then t[rowidx] = {}; end
						t[rowidx][colidx] = { text = format("|cffff0000Unusable: %s|r", msg), align = "CENTER" };
					end
				else
					-- identical item
					rowidx = rowidx + 1;
					if (t[rowidx] == nil) then t[rowidx] = {}; end
					t[rowidx][colidx] = { text = "|cfff0c501Identical|r", align = "CENTER" };
				end
			else
				-- no item found, add blank
				if (t[rowidx] == nil) then t[rowidx] = { } end
				t[rowidx][colidx] = { text = "|cff999999---|r", align = "CENTER" };
			end
		end
		
		-- set rowstart to 
		rowstart = #t;
	end

	-- add column count
	t.cols = #sortedNames;
	
	-- add title cell
	t[1][1] = { text = "|cfff0c501Team Equip Compare|r", align = "LEFT", colspan = t.cols };

	return t;
end

-- TODO: figure out enchantments!
function TEC_GetSlotCompareData(unit, ttData, slotcount, slotidx)
	local data = { stats = { } };

	-- grab item from syncronized party member data
	local link = unit.gear[ttData.slot.ids[slotidx]];
	if (link) then
		-- set item display text
		data.text = TEC_GetItemText(link, slotcount, slotidx, ttData.slot.showType);
		if (data.text) then
			data.stats = TEC_GetItemComparisonStats(unit, ttData, link)
		end
	end

	return data;
end

function TEC_GetItemText(link, slotcount, slotidx, showType)
	local name, _, quality, ilevel, _, class, subclass, _, equipSlot, _, _ = GetItemInfo(link);
	if (name) then
		local itemSlot = TEC_Slot_Lookup[equipSlot];
		local showSlotNum = (slotcount > 1 and (class ~= "Weapon" and itemSlot.text ~= "Off Hand"));
		local friendlySubclass = TEC_ItemSubclass_Lookup[subclass];

		if (showSlotNum and showType) then
			return format("|c%s[%d] %s %d: %s|r", TEC_QualityColor_Lookup[quality], ilevel, itemSlot.text, slotidx, friendlySubclass);
		elseif (showSlotNum) then
			return format("|c%s[%d] %s %d|r", TEC_QualityColor_Lookup[quality], ilevel, itemSlot.text, slotidx);
		elseif (showType) then
			return format("|c%s[%d] %s: %s|r", TEC_QualityColor_Lookup[quality], ilevel, itemSlot.text, friendlySubclass);
		else
			return format("|c%s[%d] %s|r", TEC_QualityColor_Lookup[quality], ilevel, itemSlot.text);
		end
	end
end

function TEC_GetItemComparisonStats(unit, ttData, link)
	-- check if item is usable
	local stats = { };

	-- add stat comparison
	local statDelta = GetItemStatDelta(ttData.link, link);
	for stat, value in pairs(statDelta) do
		-- get friendly name string from lookup table
		statName = TEC_Stat_Lookup[stat];
		if (not statName) then statName = tostring(stat) end

		-- color comparison text depending on +/-
		if (value > 0) then
			stats[#stats + 1] = format("|cff00ff00+%d|r %s", value, statName);
		elseif (value < 0) then
			stats[#stats + 1] = format("|cffff0000%d|r %s", value, statName);
		end
	end

	return stats;
end

function TEC_GetTooltipItemData(gametip)
	local t = {};

	t.name, t.link = gametip:GetItem();
	_, _, _, _, t.reqLevel, t.class, t.subclass, _, t.equipSlot, _, _ = GetItemInfo(t.link);
	t.slot = TEC_Slot_Lookup[t.equipSlot];
	
	return t;
end

function TEC_IsItemUsable(unit, class, reqLevel, itemClass, itemSubclass)
	local msg = "";

	if (not tec.db.global.hideUnusable) then
		msg = "";
	elseif (tec.db.global.levelCheck) and (reqLevel > unit.level) then
		msg = "Level "..reqLevel;
	elseif (not class) then
		msg = "Invalid";
	elseif (not class[itemClass]) then
		msg = itemSubclass;
	elseif (not class[itemClass][itemSubclass]) then
		msg = itemSubclass;
	elseif (tec.db.global.levelCheck) and (class[itemClass][itemSubclass] > unit.level) then
		msg = "Level "..class[itemClass][itemSubclass];
	end

	return (#msg == 0), msg;
end

function TEC_IsTooltipActive()
	-- check if the line has been added already
	if (tec_lineAdded) then return false end

	-- check if we require ctrl key
	if (not tec.db.global.requireCtrlKey) then return true end
	
	-- otherwise, check if ctrl key is pressed
	if (IsControlKeyDown()) then return true end

	-- if we got here, it's not active
	return false;
end

function TEC_DataIsComplete(unit)
	-- make sure we have all the data we need
	-- (there is sometimes a little lag/hiccup resulting in errors)
	if (not unit)
			or (not unit.name)
			or (not unit.class)
			or (not unit.level)
			or (not unit.spec)
			or not (unit.gear)
			or (not unit.ilevel)
			or (not unit.ilevel.equipped) then
		return false;
	end
	
	return true;
end

-- hook into the game tooltip
GameTooltip:HookScript("OnTooltipSetItem", TEC_OnTooltipSetItem);
GameTooltip:HookScript("OnTooltipCleared", TEC_OnTooltipCleared);
